@@ -38,7 +38,7 @@ from xmlutil import *
 def cmd_end(cmd,dir = ".."):
     "Go up one level."
     levels.droplevel()
-def cmd_exit(cmd):
+def cmd_exit(cmd,rc = 0):
     "Exit the crm program"
     cmd_end(cmd)
     if options.interactive and not options.batch:
@@ -49,7 +49,7 @@ def cmd_exit(cmd):
         pass
     for f in vars.tmpfiles:
         os.unlink(f)
-    sys.exit()
+    sys.exit(rc)
 
 class UserInterface(object):
     '''
@@ -139,13 +139,6 @@ class CliOptions(UserInterface):
         if no_questions_asked and not options.interactive:
             self.save_options("save")
 
-def listshadows():
-    return stdout2list("ls %s | fgrep shadow. | sed 's/^shadow\.//'" % vars.crm_conf_dir)
-def shadowfile(name):
-    return "%s/shadow.%s" % (vars.crm_conf_dir, name)
-def shadow2doc(name):
-    return file2doc(shadowfile(name))
-
 class CibShadow(UserInterface):
     '''
     CIB shadow management class
@@ -174,14 +167,17 @@ class CibShadow(UserInterface):
             return False
         return True
     def new(self,cmd,name,*args):
-        "usage: new <shadow_cib> [withstatus] [force]"
+        "usage: new <shadow_cib> [withstatus] [force] [empty]"
         if not is_filename_sane(name):
             return False
-        new_cmd = "%s -c '%s'" % (self.extcmd,name)
         for par in args:
-            if not par in ("force","--force","withstatus"):
+            if not par in ("force","--force","withstatus","empty"):
                 syntax_err((cmd,name,par), context = 'new')
                 return False
+        if "empty" in args:
+            new_cmd = "%s -e '%s'" % (self.extcmd,name)
+        else:
+            new_cmd = "%s -c '%s'" % (self.extcmd,name)
         if user_prefs.get_force() or "force" in args or "--force" in args:
             new_cmd = "%s --force" % new_cmd
         if ext_cmd(new_cmd) == 0:
@@ -669,9 +665,6 @@ def rm_meta_attribute(node,attr,l):
     Build a list of nvpair nodes which contain attribute
     (recursively in all children resources)
     '''
-    if node.tagName == "group":
-        # don't go into groups
-        return
     for c in node.childNodes:
         if not is_element(c):
             continue
@@ -679,7 +672,7 @@ def rm_meta_attribute(node,attr,l):
             nvpair = get_attr_in_set(c,attr)
             if nvpair:
                 l.append(nvpair)
-        elif is_child_rsc(c):
+        elif is_child_rsc(c) and not c.parentNode.tagName == "group":
             rm_meta_attribute(c,attr,l)
 def clean_inferior_attributes(node,attr):
     'Remove attr from all resources below'
@@ -698,6 +691,7 @@ def set_deep_meta_attr(attr,value,rsc_id):
     '''
     target_node = rsc2node(rsc_id)
     if not target_node:
+        common_error("resource %s does not exist" % rsc_id)
         return False
     if not (target_node.tagName == "primitive" and \
         target_node.parentNode.tagName == "group"):
@@ -714,18 +708,11 @@ def get_max_clone(id):
     except:
         cnt = len(listnodes())
     return cnt
-def cleanup_resource(rsc,node):
+def cleanup_resource(rsc,node = ''):
     if not is_name_sane(rsc) or not is_name_sane(node):
         return False
-    if is_rsc_clone(rsc) or is_rsc_ms(rsc):
-        base = get_cloned_rsc(rsc)
-        if not base:
-            return False
-        clone_max = get_max_clone(rsc)
-        rc = True
-        for n in range(clone_max):
-            if ext_cmd(RscMgmt.rsc_cleanup % ("%s:%d" % (base,n), node)) != 0:
-                rc = False
+    if not node:
+        rc = ext_cmd(RscMgmt.rsc_cleanup_all%(rsc)) == 0
     else:
         rc = ext_cmd(RscMgmt.rsc_cleanup%(rsc,node)) == 0
     return rc
@@ -741,6 +728,7 @@ class RscMgmt(UserInterface):
     rsc_migrate = "crm_resource -M -r '%s' %s"
     rsc_unmigrate = "crm_resource -U -r '%s'"
     rsc_cleanup = "crm_resource -C -r '%s' -H '%s'"
+    rsc_cleanup_all = "crm_resource -C -r '%s'"
     rsc_param =  {
         'set': "crm_resource -r '%s' -p '%s' -v '%s'",
         'delete': "crm_resource -r '%s' -d '%s'",
@@ -771,7 +759,7 @@ class RscMgmt(UserInterface):
         self.cmd_table["demote"] = (self.demote,(1,1),0,(rsc_list,))
         self.cmd_table["manage"] = (self.manage,(1,1),0,(rsc_list,))
         self.cmd_table["unmanage"] = (self.unmanage,(1,1),0,(rsc_list,))
-        self.cmd_table["migrate"] = (self.migrate,(1,3),0,(rsc_list,nodes_list))
+        self.cmd_table["migrate"] = (self.migrate,(1,4),0,(rsc_list,nodes_list))
         self.cmd_table["unmigrate"] = (self.unmigrate,(1,1),0,(rsc_list,))
         self.cmd_table["param"] = (self.param,(3,4),1,(rsc_list,attr_cmds))
         self.cmd_table["meta"] = (self.meta,(3,4),1,(rsc_list,attr_cmds))
@@ -802,8 +790,12 @@ class RscMgmt(UserInterface):
         "usage: restart <rsc>"
         if not is_name_sane(rsc):
             return False
+        common_info("ordering %s to stop" % rsc)
         if not self.stop("stop",rsc):
             return False
+        if not wait4dc("stop", not options.batch):
+            return False
+        common_info("ordering %s to start" % rsc)
         return self.start("start",rsc)
     def stop(self,cmd,rsc):
         "usage: stop <rsc>"
@@ -837,27 +829,47 @@ class RscMgmt(UserInterface):
             return False
         return set_deep_meta_attr("is-managed","false",rsc)
     def migrate(self,cmd,*args):
-        """usage: migrate <rsc> [<node>] [<lifetime>]"""
+        """usage: migrate <rsc> [<node>] [<lifetime>] [force]"""
         rsc = args[0]
         if not is_name_sane(rsc):
             return False
         node = None
         lifetime = None
+        force = False
         if len(args) == 2:
             if not args[1] in listnodes():
-                lifetime = args[1]
+                if args[1] == "force":
+                    force = True
+                else:
+                    lifetime = args[1]
             else:
                 node = args[1]
         elif len(args) == 3:
+            if not args[1] in listnodes():
+                lifetime = args[1]
+                force = True
+            else:
+                node = args[1]
+                if args[2] == "force":
+                    force = True
+                else:
+                    lifetime = args[2]
+        elif len(args) == 4:
             if not is_name_sane(args[1]):
                 return False
             node = args[1]
             lifetime = args[2]
+            if not args[3] == "force":
+                syntax_err((cmd,force))
+                return False
+            force = True
         opts = ''
         if node:
             opts = "--node='%s'" % node
         if lifetime:
             opts = "%s --lifetime='%s'" % (opts,lifetime)
+        if force or user_prefs.get_force():
+            opts = "%s --force" % opts
         return ext_cmd(self.rsc_migrate % (rsc,opts)) == 0
     def unmigrate(self,cmd,rsc):
         "usage: unmigrate <rsc>"
@@ -871,11 +883,7 @@ class RscMgmt(UserInterface):
         if len(args) == 2: # remove
             return cleanup_resource(args[0],args[1])
         else:
-            rv = True
-            for n in listnodes():
-                if not cleanup_resource(args[0],n):
-                    rv = False
-            return rv
+            return cleanup_resource(args[0])
     def failcount(self,cmd,*args):
         """usage:
         failcount <rsc> set <node> <value>
@@ -938,6 +946,7 @@ class NodeMgmt(UserInterface):
     node_standby = "crm_standby -N '%s' -v '%s' %s"
     node_delete = "cibadmin -D -o nodes -X '<node uname=\"%s\"/>'"
     node_delete_status = "cibadmin -D -o status -X '<node_state uname=\"%s\"/>'"
+    node_clear_state = "cibadmin %s -o status --xml-text '<node_state id=\"%s\" uname=\"%s\" ha=\"active\" in_ccm=\"false\" crmd=\"offline\" join=\"member\" expected=\"down\" crm-debug-origin=\"manual_clear\" shutdown=\"0\"/>'"
     hb_delnode = "/usr/share/heartbeat/hb_delnode '%s'"
     crm_node = "crm_node"
     node_fence = "crm_attribute -t status -U '%s' -n terminate -v true"
@@ -961,6 +970,7 @@ class NodeMgmt(UserInterface):
         self.cmd_table["online"] = (self.online,(0,1),0,(nodes_list,))
         self.cmd_table["fence"] = (self.fence,(1,1),0,(nodes_list,))
         self.cmd_table["delete"] = (self.delete,(1,1),0,(nodes_list,))
+        self.cmd_table["clearstate"] = (self.clearstate,(1,1),0,(nodes_list,))
         self.cmd_table["attribute"] = (self.attribute,(3,4),0,(nodes_list,attr_cmds))
         self.cmd_table["status-attr"] = (self.status_attr,(3,4),0,(nodes_list,attr_cmds))
         self.cmd_aliases.update({
@@ -1045,11 +1055,26 @@ class NodeMgmt(UserInterface):
             node = vars.this_node
         if not is_name_sane(node):
             return False
+        if not user_prefs.get_force() and \
+                not ask("Do you really want to shoot %s?" % node):
+            return False
         return ext_cmd(self.node_fence%(node)) == 0
+    def clearstate(self,cmd,node):
+        'usage: clearstate <node>'
+        if not is_name_sane(node):
+            return False
+        if not user_prefs.get_force() and \
+                not ask("Do you really want to drop state for node %s?" % node):
+            return False
+        return ext_cmd(self.node_clear_state%("-M -c",node,node)) == 0 and \
+            ext_cmd(self.node_clear_state%("-R",node,node)) == 0
     def delete(self,cmd,node):
         'usage: delete <node>'
         if not is_name_sane(node):
             return False
+        if not node in listnodes():
+            common_err("node %s not found in the CIB" % node)
+            return False
         rc = True
         if cluster_stack() == "heartbeat":
             rc = ext_cmd(self.hb_delnode%node) == 0
@@ -1065,18 +1090,28 @@ class NodeMgmt(UserInterface):
                 if a[1] == node:
                     node_states[a[2]] = 1
                     if a[2] == "lost":
-                        if ext_cmd("%s --force -R %s" % (self.crm_node,a[0])) != 0:
-                            rc = False
-            if not "lost" in node_states:
-                common_err('node %s/state "lost" not found in the id list' % node)
-                if "member" in node_states:
-                    common_info("node %s appears to be still active" % node)
+                        ec = ext_cmd("%s --force -R %s" % (self.crm_node,a[0]))
+                        if ec != 0:
+                            common_warn('"%s --force -R %s" failed, rc=%d' % (self.crm_node,a[0],ec))
+            if not node_states:
+                common_info("node %s not found by %s" % (node,self.crm_node))
+            elif "member" in node_states:
+                common_info("node %s appears to be still active" % node)
+                common_info("check output of %s -l" % self.crm_node)
+                rc = False
+            elif not "lost" in node_states:
+                common_err("node %s's state not recognized: %s" % (node,node_states.keys()))
                 common_info("check output of %s -l" % self.crm_node)
                 rc = False
         if rc:
             if ext_cmd(self.node_delete%node) != 0 or \
                     ext_cmd(self.node_delete_status%node) != 0:
+                common_err("failed to remove %s from the CIB" % node)
                 rc = False
+        if rc:
+            common_info("node %s deleted" % node)
+        else:
+            common_err('failed to delete node %s' % node)
         return rc
     def attribute(self,cmd,*args):
         """usage:
@@ -1103,7 +1138,7 @@ class RA(UserInterface):
         self.help_table = help_sys.load_level("ra")
         self.cmd_table["classes"] = (self.classes,(0,0),0)
         self.cmd_table["list"] = (self.list,(1,2),1)
-        self.cmd_table["providers"] = (self.providers,(1,1),1)
+        self.cmd_table["providers"] = (self.providers,(1,2),1)
         self.cmd_table["meta"] = (self.meta,(1,3),1)
         self.cmd_aliases.update({
             "meta": ("info",),
@@ -1116,9 +1151,9 @@ class RA(UserInterface):
                 print "%s / %s" % (c,' '.join(ra_providers_all(c)))
             else:
                 print "%s" % c
-    def providers(self,cmd,ra_type):
-        "usage: providers <ra>"
-        print ' '.join(ra_providers(ra_type))
+    def providers(self,cmd,ra_type,ra_class = "ocf"):
+        "usage: providers <ra> [<class>]"
+        print ' '.join(ra_providers(ra_type,ra_class))
     def list(self,cmd,c,p = None):
         "usage: list <class> [<provider>]"
         if not c in ra_classes():
